#property strict

class ARRAYRESIZE
{
  static string GetTickFlag( uint tickflag )
  {
    string flag = " " + (string)tickflag;

  #define TICKFLAG_MACRO(A) flag += ((bool)(tickflag & TICK_FLAG_##A)) ? " TICK_FLAG_" + #A : ""; \
                            tickflag -= tickflag & TICK_FLAG_##A;
    TICKFLAG_MACRO(BID)
    TICKFLAG_MACRO(ASK)
    TICKFLAG_MACRO(LAST)
    TICKFLAG_MACRO(VOLUME)
    TICKFLAG_MACRO(BUY)
    TICKFLAG_MACRO(SELL)
  #undef TICKFLAG_MACRO

    if (tickflag)
      flag += " FLAG_UNKNOWN (" + (string)tickflag + ")";

    return(flag);
  }

  #define TOSTR(A) " " + #A + " = " + (string)Tick.A
  #define TOSTR2(A) " " + #A + " = " + ::DoubleToString(Tick.A, _Digits)

  static string TickToString( const MqlTick &Tick )
  {
    return(TOSTR(time) + "." + ::IntegerToString(Tick.time_msc % 1000, 3, '0') +
           TOSTR2(bid) + TOSTR2(ask) + TOSTR2(last)+ TOSTR(volume) + ARRAYRESIZE::GetTickFlag(Tick.flags));
  }

  #undef TOSTR2
  #undef TOSTR

  template <typename T>
  static T MyPrint( const T Value, const string Str )
  {
    ::Print(Str + " = " + (string)Value);

    return(Value);
  }

public:

#define _P(A) ARRAYRESIZE::MyPrint(A, __FUNCSIG__ + ", Line = " + (string)__LINE__ + ": " + #A)

  template <typename T>
  static int ArrayResize_Real( T &Array[], const int NewSize, const int Reserve = 0 )
  {
    ::ResetLastError();

    const int PrevSize = ::ArraySize(Array);

    const ulong StartTime = ::GetMicrosecondCount();
    const int Res = ::ArrayResize(Array, NewSize, Reserve);
    const ulong DuringTime = ::GetMicrosecondCount() - StartTime;

    if (::GetLastError() || (Res == -1) || (Res != NewSize) || (NewSize != ::ArraySize(Array)))
    {
      ::Alert(::MQLInfoString(MQL_PROGRAM_NAME) + " - ERROR in " + __FUNCSIG__ + "! - see the logs.");

      _P(::MQLInfoString(MQL_PROGRAM_PATH));
      ::Print(::EnumToString((ENUM_PROGRAM_TYPE)::MQLInfoInteger(MQL_PROGRAM_TYPE)));
      _P(typename(T));
      _P(sizeof(T));
      _P(Res);
      _P(NewSize);
      _P(Reserve);
      _P(::ArraySize(Array));
      _P(PrevSize);
      _P(NewSize * sizeof(T));
      _P(PrevSize * sizeof(T));
      _P(Reserve * sizeof(T));
      _P(::GetLastError());
      _P(DuringTime);

      _P(::TerminalInfoInteger(TERMINAL_BUILD));
      _P(::TerminalInfoInteger(TERMINAL_DISK_SPACE));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_PHYSICAL));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_AVAILABLE));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_TOTAL));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_USED));

      _P(::MQLInfoInteger(MQL_MEMORY_USED));
      _P(::MQLInfoInteger(MQL_MEMORY_LIMIT));

      _P(::ChartID());
      _P(::PeriodSeconds());
      _P(::Symbol());
      _P(::TimeCurrent());
      _P(::TimeLocal());

      MqlTick Tick = {0};

      _P(::SymbolInfoTick(_Symbol, Tick));
      ::Print(ARRAYRESIZE::TickToString(Tick));

      _P(::ChartSetInteger(0, CHART_BRING_TO_TOP, true));

      ::DebugBreak();
    }

    return(Res);
  }

  // https://www.mql5.com/ru/forum/363680
  template <typename T>
  static int ArrayResize_Real( T &Array[][], const int NewSize, const int Reserve = 0 )
  {
    ::ResetLastError();

    const int PrevSize = ::ArraySize(Array);

    const ulong StartTime = ::GetMicrosecondCount();
    const int Res = ::ArrayResize(Array, NewSize, Reserve);
    const ulong DuringTime = ::GetMicrosecondCount() - StartTime;

    if (::GetLastError() || (Res == -1) ||
      #ifdef __MQL5__
        (Res != NewSize)
      #else // #ifdef __MQL5__
        (Res != NewSize * ::ArrayRange(Array, 1))
      #endif // #ifdef __MQL5__ #else
        )
    {
      ::Alert(::MQLInfoString(MQL_PROGRAM_NAME) + " - ERROR in " + __FUNCSIG__ + "! - see the logs.");

      _P(::MQLInfoString(MQL_PROGRAM_PATH));
      ::Print(::EnumToString((ENUM_PROGRAM_TYPE)::MQLInfoInteger(MQL_PROGRAM_TYPE)));
      _P(typename(T));
      _P(sizeof(T));
      _P(Res);
      _P(NewSize);
      _P(NewSize * ::ArrayRange(Array, 1));
      _P(Reserve);
      _P(::ArraySize(Array));
      _P(::ArrayRange(Array, 0));
      _P(::ArrayRange(Array, 1));
      _P(PrevSize);
      _P(NewSize * sizeof(T));
      _P(PrevSize * sizeof(T));
      _P(Reserve * sizeof(T));
      _P(::GetLastError());
      _P(DuringTime);

      _P(::TerminalInfoInteger(TERMINAL_BUILD));
      _P(::TerminalInfoInteger(TERMINAL_DISK_SPACE));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_PHYSICAL));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_AVAILABLE));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_TOTAL));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_USED));

      _P(::MQLInfoInteger(MQL_MEMORY_USED));
      _P(::MQLInfoInteger(MQL_MEMORY_LIMIT));

      _P(::ChartID());
      _P(::PeriodSeconds());
      _P(::Symbol());
      _P(::TimeCurrent());
      _P(::TimeLocal());

      MqlTick Tick = {0};

      _P(::SymbolInfoTick(_Symbol, Tick));
      ::Print(ARRAYRESIZE::TickToString(Tick));

      _P(::ChartSetInteger(0, CHART_BRING_TO_TOP, true));

      ::DebugBreak();
    }

    return(Res);
  }
#undef ARRAY_RANGE

  template <typename T, typename T2>
  static int ArrayCopy_Real( T &Dst_Array[], const T2 &Src_Array[], const int Dst_Start = 0, const int Src_Start = 0, const int Count = WHOLE_ARRAY )
  {
    const int PrevSize = ::ArraySize(Dst_Array);

    ::ResetLastError();

    const ulong StartTime = ::GetMicrosecondCount();
    const int Res = ::ArrayCopy(Dst_Array, Src_Array, Dst_Start, Src_Start, Count);
    const ulong DuringTime = ::GetMicrosecondCount() - StartTime;

  #ifdef __MQL5__
    if (::GetLastError() == ERR_ZEROSIZE_ARRAY)
      ::ResetLastError();
  #endif // __MQL5__

    const bool Tmp = ((Count == WHOLE_ARRAY) || (Count < 0) || (Count > ::ArraySize(Src_Array) - Src_Start));

    if (::GetLastError() || (Tmp && (Res != ::ArraySize(Src_Array) - Src_Start)) ||
        (!Tmp && (Res != Count)) || (typename(T) != typename(T2)))
    {
      ::Alert(::MQLInfoString(MQL_PROGRAM_NAME) + " - ERROR in " + __FUNCSIG__ + "! - see the logs.");

      _P(::MQLInfoString(MQL_PROGRAM_PATH));
      ::Print(::EnumToString((ENUM_PROGRAM_TYPE)::MQLInfoInteger(MQL_PROGRAM_TYPE)));
      _P(typename(T));
      _P(typename(T2));
      _P(sizeof(T));
      _P(sizeof(T2));
      _P(Res);
      _P(::ArraySize(Dst_Array));
      _P(::ArraySize(Src_Array));
      _P(Dst_Start);
      _P(Src_Start);
      _P(Count);
      _P(::ArraySize(Src_Array) - Src_Start);
      _P(PrevSize);
      _P(::ArraySize(Dst_Array) * sizeof(T));
      _P(PrevSize * sizeof(T));
      _P(::GetLastError());
      _P(DuringTime);

      _P(::TerminalInfoInteger(TERMINAL_BUILD));
      _P(::TerminalInfoInteger(TERMINAL_DISK_SPACE));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_PHYSICAL));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_AVAILABLE));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_TOTAL));
      _P(::TerminalInfoInteger(TERMINAL_MEMORY_USED));

      _P(::MQLInfoInteger(MQL_MEMORY_USED));
      _P(::MQLInfoInteger(MQL_MEMORY_LIMIT));

      _P(::ChartID());
      _P(::PeriodSeconds());
      _P(::Symbol());
      _P(::TimeCurrent());
      _P(::TimeLocal());

      MqlTick Tick = {0};

      _P(::SymbolInfoTick(_Symbol, Tick));
      ::Print(ARRAYRESIZE::TickToString(Tick));

      _P(::ChartSetInteger(0, CHART_BRING_TO_TOP, true));

      ::DebugBreak();
    }

    return(Res);
  }

#undef _P
};

#define ArrayResize ARRAYRESIZE::ArrayResize_Real
#define ArrayCopy   ARRAYRESIZE::ArrayCopy_Real